<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */

namespace FiloBlu\Esb\Model\Rest;
use FiloBlu\Esb\Api\ExternalInvoiceInterface;
use Magento\Sales\Api\Data\InvoiceInterface;
use Magento\Framework\Filesystem\DriverPool;
if (!defined("DS")):
      define("DS", DIRECTORY_SEPARATOR);
endif;

/**
 * REST Api to save external invoices
 */
class ExternalInvoice implements ExternalInvoiceInterface{
	
    /**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */ 	
    protected $_orderRepository;
	
    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder 
     */ 
    protected $_searchCriteriaBuilder;
	
    /**
     * @var array
     */ 	
    protected $_standardPaths = array();

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

	/**
     * @var \Magento\Framework\Filesystem\Directory\WriteFactory
     */     
    protected $_writeFactory;

	/**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */ 
    protected $_orderExtensionFactory;

    /**
     * Init plugin
     *
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\App\Filesystem\DirectoryList $directoryList
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory
     * @param \Magento\Framework\Filesystem\Directory\WriteFactory $writeFactory
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder     
     */
    public function __construct(
            \Magento\Framework\App\Helper\Context $context,
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
            \Magento\Framework\App\Filesystem\DirectoryList $directoryList,
            \Magento\Sales\Model\OrderRepository $orderRepository,
            \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
            \Magento\Framework\Filesystem\Directory\WriteFactory $writeFactory,
            \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
    ){
        $this->_scopeConfig = $scopeConfig;
        $this->_standardPaths["base_dir"] = rtrim($directoryList->getRoot(), DS).DS;
        $this->_standardPaths["media_dir"] = rtrim($directoryList->getPath('media'), DS).DS;
        $this->_standardPaths["pub_dir"] = rtrim($directoryList->getPath('pub'), DS).DS;
        $this->_standardPaths["var_dir"] = rtrim($directoryList->getPath('var'), DS).DS;
        $this->_standardPaths["tmp_dir"] = rtrim($directoryList->getPath('tmp'), DS).DS;
        $this->_orderRepository = $orderRepository;
        $this->_orderExtensionFactory = $orderExtensionFactory;
        $this->_writeFactory = $writeFactory;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;       
    }

    /**
     * Save External Invoice data. Save PDF file and additional data on Sales Order.
     *
     * @param \Magento\Sales\Api\Data\InvoiceInterface  $invoice_data
     * @return int
     */
     public function save(InvoiceInterface $invoice_data){
        $invoiceExtensionAttributes=$invoice_data->getExtensionAttributes();
        
        if(null==$invoiceExtensionAttributes || !$invoiceExtensionAttributes->getOrderIncrementId() || !$invoiceExtensionAttributes->getDocumentType()){
            return 0;
        }
        if(null==$invoice_data->getIncrementId()){
            return 0;
        }
        $filename="order_".$invoiceExtensionAttributes->getOrderIncrementId()."_".$invoiceExtensionAttributes->getDocumentType()."_".$invoice_data->getIncrementId().".pdf";

        $document_folder=$this->_scopeConfig->getValue(
                'esb/print_documents/document_folder', 
                \Magento\Store\Model\ScopeInterface::SCOPE_STORE
            );
        foreach($this->_standardPaths as $key => $value){
            $document_folder=str_replace("{{".$key."}}", $value,  $document_folder);
        }
        error_log($document_folder);
        $writer=$this->_writeFactory->create($document_folder, DriverPool::FILE);
        /** @var \Magento\Framework\Filesystem\File\WriteInterface|\Magento\Framework\Filesystem\File\Write $file */
        
        $file = $writer->openFile($filename, 'w');
        try {
            $file->lock();
            try {
                $file->write(base64_decode($invoiceExtensionAttributes->getDocumentPdf()));
            }
            finally {
                $file->unlock();
            }
        }
        finally {
            $file->close();
        }

        $invoice=array(
            "increment_id" => $invoice_data->getIncrementId(),
            "document_type" => $invoiceExtensionAttributes->getDocumentType(),
            "grand_total" => $invoice_data->getGrandTotal(),
            "document_pdf" => $filename
        );


        $searchCriteria = $this->_searchCriteriaBuilder
            ->addFilter('increment_id', $invoiceExtensionAttributes->getOrderIncrementId(), 'eq')->create();
        $orderList = $this->_orderRepository->getList($searchCriteria);
        
        if(count($orderList)<>1){
            return 0;
        }
        try{
            $order=$orderList->getFirstItem();
            $orderExtensionAttributes=$order->getExtensionAttributes();
            $additionalData=array("external_invoices" => array($invoice_data->getIncrementId() => $invoice));
            if($orderExtensionAttributes){
                if($orderExtensionAttributes->getFbAdditionalData()){
                    $additionalData=current($orderExtensionAttributes->getFbAdditionalData());
                    if(array_key_exists("external_invoices", $additionalData)){
                        $additionalData["external_invoices"][$invoice_data->getIncrementId()]=$invoice;
                    }else{
                        $additionalData["external_invoices"]=array($invoice_data->getIncrementId() => $invoice);
                    }
                }
            }
            else{
                $orderExtensionAttributes=$this->_orderExtensionFactory->create();
            }
            $orderExtensionAttributes->setFbAdditionalData(array($additionalData));
            $order->setExtensionAttributes($orderExtensionAttributes);
            $this->_orderRepository->save($order);
            return $order->getId();
        }catch(Exception $e){
            return 0;
        }
    }
}

